import { CoreEntity } from "src/core/core.entity";
import { AfterInsert, Column, Entity, getConnection, OneToMany } from "typeorm";
import { OrderDetails } from "../../order-details/entities/order-details.entity";

@Entity('orders')
export class Order extends CoreEntity {
    @Column({ unique: true })
    orderId: string;

    @Column()
    customerId: string;

    @Column()
    customerName: string;

    @Column({ default: null })
    couponNumber: string;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0 })
    couponDiscount: number;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0 })
    totalQuantity: number;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0 })
    subTotal: number;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0 })
    total: number;

    @Column({ default: null, type: "longtext" })
    shippingAddress: string;

    @Column({ default: null, type: "longtext" })
    billingAddress: string;

    @Column({ default: "Cash on Delivery" })
    paymentMethodName: string;

    @Column({ default: "PENDING" })
    paymentStatus: string;

    @Column({ default: "PENDING" })
    status: string;

    @Column()
    createdBy: string;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0 })
    shippingCost: number;

    @Column({ default: null })
    shippingType: string;

    @Column({ default: null })
    serviceType: string;

    @Column({ default: null })
    trackingNumber: string;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0, nullable: true })
    totalWeight: number;

    @Column({ default: null })
    discountType: string;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0, nullable: true })
    discountTotal: number;

    @Column({ default: null, nullable: true })
    salesTaxTitle: string;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0, nullable: true })
    salesTaxRate: number;

    @Column({ type: "decimal", precision: 10, scale: 2, default: 0, nullable: true })
    salesTaxAmount: number;

    @Column({ default: '************1111', nullable: true })
    cardNumber: string;

    @Column({ default: null, nullable: true })
    labelStockType: string;

    @Column({ default: null, nullable: true })
    labelImageType: string;

    @Column({ default: null, nullable: true, type: 'longtext' })
    labelImageEncode: string;

    @OneToMany(() => OrderDetails, orderDetails => orderDetails.order, { nullable: true })
    orders: OrderDetails[];

    @AfterInsert()
    processInvoiceService() {
        if (this.couponNumber != null) {
            let sql = "CALL cuponUsedWithInvoice('" + this.couponNumber + "')";
            getConnection().query(sql);
        }
    }
}
