import { Between, EntityRepository, FindOperator, getConnection, Repository } from "typeorm";
import { PaymentGateway } from "./entities/payment-gateway.entity";


@EntityRepository(PaymentGateway)
export class PaymentGatewayRepository extends Repository<PaymentGateway> {
    async createEntity(
        dto: any
    ): Promise<any | Error> {
        try {
            const entity = this.create(dto);
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(
        id: string
    ): Promise<PaymentGateway | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(
        id: string,
        dto: any
    ): Promise<PaymentGateway | Error> {
        try {
            await this.update(id, {
                refundTransID: dto.refundTransID,
                refundPayload: dto.refundPayload,
                isRefunded: dto.isRefunded,
                refundDate: dto.refundDate,
                refundAmount: dto.refundAmount,
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<PaymentGateway[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(PaymentGateway)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async findAllEntities(query: any): Promise<any | Error> {
        try {
            let strstartDate: any;
            let strendDate: any;
            type FindOption = {
                paymentTransID?: FindOperator<string>,
                paymentDate?: FindOperator<Date>,
                customerName?: FindOperator<string>,
                invoiceID?: FindOperator<string>,
            }
            const findOptions: FindOption = {};

            if (query.paymentTransID) findOptions.paymentTransID = query.paymentTransID;
            if (query.customerName) findOptions.customerName = query.customerName;
            if (query.invoiceID) findOptions.invoiceID = query.invoiceID;
            if (query.startDate) strstartDate = query.startDate;
            if (query.endDate) strendDate = query.endDate;

            const startDate = new Date(strstartDate);
            startDate.setUTCHours(0, 0, 0, 0);
            const endDate = new Date(strendDate);
            endDate.setUTCHours(23, 59, 59, 999);

            if (query.startDate && query.endDate) findOptions.paymentDate = Between(
                new Date(startDate),
                new Date(endDate),
            );

            const data = await this.find({
                order: { createdAt: -1 },
                where: {
                    ...findOptions,
                    isDeleted: false
                },
            });

            return data;
        } catch (e) {
            return e;
        }
    }

    async refundReportEntities(query: any): Promise<any | Error> {
        try {
            let strstartDate: any;
            let strendDate: any;
            type FindOption = {
                refundTransID?: FindOperator<string>,
                refundDate?: FindOperator<Date>,
                customerName?: FindOperator<string>,
                invoiceID?: FindOperator<string>,
            }
            const findOptions: FindOption = {};

            if (query.refundTransID) findOptions.refundTransID = query.refundTransID;
            if (query.customerName) findOptions.customerName = query.customerName;
            if (query.invoiceID) findOptions.invoiceID = query.invoiceID;
            if (query.startDate) strstartDate = query.startDate;
            if (query.endDate) strendDate = query.endDate;

            const startDate = new Date(strstartDate);
            startDate.setUTCHours(0, 0, 0, 0);
            const endDate = new Date(strendDate);
            endDate.setUTCHours(23, 59, 59, 999);

            if (query.startDate && query.endDate) findOptions.refundDate = Between(
                new Date(startDate),
                new Date(endDate),
            );

            const data = await this.find({
                where: {
                    ...findOptions,
                    isRefunded: true,
                    isDeleted: false
                },
            });

            return data;
        } catch (e) {
            return e;
        }
    }

    async checkOrderIsRefunded(orderId: string, productId: string): Promise<any | Error> {
        try {
            return await this.findOne({
                where: {
                    refundProductID: productId,
                    invoiceID: orderId,
                    isRefunded: true,
                    isDeleted: false
                },
            });
        } catch (e) {
            return e;
        }
    }
}