import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateProductColorVariantsDto } from "./dto/create-product-color-variants.dto";
import { UpdateProductColorVariantsDto } from "./dto/update-product-color-variants.dto";
import { ProductColorVariants } from "./entities/product-color-variants.entity";

@EntityRepository(ProductColorVariants)
export class ProductColorVariantsRepository extends Repository<ProductColorVariants> {

    async createEntity(dto: CreateProductColorVariantsDto): Promise<ProductColorVariants | Error> {
        try {
            const { productId, productName, colorId, colorName, colorCode } = dto;
            const entity = this.create({
                productId,
                productName,
                colorId,
                colorName,
                colorCode
            });
            await this.save(entity);
            return entity;
        }
        catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<ProductColorVariants[] | Error> {
        try {
            return await this.find({
                order: { createdAt: "DESC" },
                where: { ...query, isDeleted: false }
            });
        }
        catch (e) {
            return e;
        }
    }

    async deleteColorVariant(id: any): Promise<any | Error> {
        try {
            const deleteResult = await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ProductColorVariants)
                .where("productId = :productId", { productId: id })
                .execute();
            if (deleteResult.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async bulkInsertColorVariant(color: any): Promise<any | Error> {
        try {
            const insertResult = await getConnection()
                .createQueryBuilder()
                .insert()
                .into(ProductColorVariants)
                .values(color)
                .execute();

            if (insertResult.raw.affected > 0) {
                return true;
            }
            return false;
        }
        catch (e) {
            return e;
        }
    }

    async updateProductColorVariant(id: any, colorEntity: any): Promise<any | Error> {
        try {
            const deleteCurrentData = await this.deleteColorVariant(id);
            await this.bulkInsertColorVariant(colorEntity)
        } catch (e) {
            return e;
        }
    }
}